import { Injectable, Component, NgModule, ErrorHandler } from '@angular/core';
import { MessageService } from 'primeng/components/common/messageservice';
import { RouterModule } from '@angular/router';
import { hasIn, isFunction, isEmpty, has, isArray } from 'lodash';
import { HttpErrorResponse } from '@angular/common/http';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ErrorHandlerService {
    /**
     * @param {?} messageService
     */
    constructor(messageService) {
        this.messageService = messageService;
        this._console = console;
    }
    /**
     * @param {?} error
     * @return {?}
     */
    handleError(error) {
        /** @type {?} */
        const errorLogger = this.getErrorLogger(error);
        /** @type {?} */
        const original = this.getOriginalError(error);
        /** @type {?} */
        const context = this.getDebugContext(error);
        /** @type {?} */
        const message = (original || error).message || `${error}`;
        errorLogger(this._console, 'ERROR', message);
        if (original) {
            errorLogger(this._console, 'ORIGINAL ERROR', original);
        }
        if (context) {
            errorLogger(this._console, 'CONTEXT', context);
        }
        this.messageService.add({
            severity: 'error',
            summary: (original || error).constructor.name,
            detail: message,
            life: 15,
            closable: true,
            data: error,
        });
    }
    /**
     * @private
     * @param {?=} error
     * @return {?}
     */
    getDebugContext(error) {
        return error ? ((/** @type {?} */ (error))).ngDebugContext || this.getDebugContext(this.getOriginalError(error)) : null;
    }
    /**
     * @private
     * @param {?} error
     * @return {?}
     */
    getOriginalError(error) {
        /** @type {?} */
        let original = error;
        while (original) {
            original = ((/** @type {?} */ (original))).ngOriginalError;
        }
        return original;
    }
    /**
     * @private
     * @param {?} error
     * @return {?}
     */
    getErrorLogger(error) {
        return ((/** @type {?} */ (error))).ngErrorLogger || this.defaultErrorLogger;
    }
    /**
     * @private
     * @param {?} console
     * @param {...?} values
     * @return {?}
     */
    defaultErrorLogger(console, ...values) {
        ((/** @type {?} */ (console.error)))(...values);
    }
}
ErrorHandlerService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ErrorHandlerService.ctorParameters = () => [
    { type: MessageService }
];
if (false) {
    /**
     * @type {?}
     * @private
     */
    ErrorHandlerService.prototype._console;
    /**
     * @type {?}
     * @private
     */
    ErrorHandlerService.prototype.messageService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class GenericErrorComponent {
}
GenericErrorComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-generic-error',
                template: "<div class=\"card ui-message ui-messages-error\">\n  <h1>\n    <i class=\"ui-messages-icon fa fa-exclamation-triangle\"></i>\n    <span i18n class=\"ui-messages-summary\">Erreur !</span>\n  </h1>\n  <p i18n class=\"ui-messages-detail\">Une erreur s'est produite !</p>\n</div>\n"
            }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NotFoundErrorComponent {
}
NotFoundErrorComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-not-found-error',
                template: "<div class=\"card ui-message ui-messages-error\">\n  <h1>\n    <i class=\"ui-messages-icon fa fa-exclamation-triangle\"></i>\n    <span i18n class=\"ui-messages-summary\">Page non trouv\u00E9e !</span>\n  </h1>\n  <p i18n class=\"ui-messages-detail\">La ressource n'a pas \u00E9t\u00E9 trouv\u00E9e.</p>\n</div>\n"
            }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const routes = [
    {
        path: 'error/404',
        pathMatch: 'full',
        component: NotFoundErrorComponent,
    },
    { path: 'error/**', component: GenericErrorComponent },
];
class ErrorModule {
}
ErrorModule.decorators = [
    { type: NgModule, args: [{
                imports: [RouterModule.forChild(routes)],
                exports: [RouterModule],
                providers: [{ provide: ErrorHandler, useClass: ErrorHandlerService }],
                declarations: [GenericErrorComponent, NotFoundErrorComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} that
 * @return {?}
 */
function isErrorMessageHandler(that) {
    return hasIn(that, 'setError') && isFunction(that.setError);
}
/**
 * @param {?} that
 * @return {?}
 */
function isConstraintViolationHandler(that) {
    return hasIn(that, 'setConstraintViolations') && isFunction(that.setConstraintViolations);
}
/**
 * @param {?} that
 * @return {?}
 */
function isValidationErrorHandler(that) {
    return hasIn(that, 'setErrors') && isFunction(that.setErrors);
}
/**
 * @param {?} that
 * @return {?}
 */
function isErrorLogger(that) {
    return hasIn(that, 'logError') && isFunction(that.logError);
}
/**
 * @param {?} that
 * @return {?}
 */
function isErrorHandler(that) {
    return (isErrorMessageHandler(that) ||
        isConstraintViolationHandler(that) ||
        isValidationErrorHandler(that) ||
        isErrorLogger(that));
}
/**
 * @param {?} that
 * @return {?}
 */
function isErrorClearer(that) {
    return hasIn(that, 'clearErrors') && isFunction(that.clearErrors);
}
/**
 * @param {?} that
 * @return {?}
 */
function isDispatchableError(that) {
    return hasIn(that, 'dispatch') && isFunction(that.dispatch);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Erreur "enveloppant" une autre erreur.
 * @template T
 */
class AppError {
    /**
     * @param {?} original
     * @param {?} message
     * @param {?=} stack
     * @param {?=} transient
     */
    constructor(original, message, stack, transient = false) {
        this.original = original;
        this.message = message;
        this.stack = stack;
        this.transient = transient;
        this.isNullError = false;
    }
    /**
     * @return {?}
     */
    get name() {
        return this.getName();
    }
    /**
     * @param {?} handler
     * @return {?}
     */
    dispatch(handler) {
        return ((isErrorLogger(handler) && handler.logError(this.message, this.name, this.stack)) ||
            (isErrorMessageHandler(handler) && handler.setError(this.message)) ||
            (isValidationErrorHandler(handler) && handler.setErrors({ message: this.message })));
    }
    /**
     * @protected
     * @return {?}
     */
    getName() {
        return (this.original instanceof Error && this.original.name) || this.constructor.name;
    }
}
if (false) {
    /** @type {?} */
    AppError.prototype.isNullError;
    /** @type {?} */
    AppError.prototype.original;
    /** @type {?} */
    AppError.prototype.message;
    /** @type {?} */
    AppError.prototype.stack;
    /** @type {?} */
    AppError.prototype.transient;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Supprime les erreurs d'un handler.
 * @param {?} handler
 * @return {?}
 */
function clearErrors(handler) {
    if (isErrorClearer(handler) && handler.clearErrors()) {
        return true;
    }
    /** @type {?} */
    let handled = false;
    if (isErrorMessageHandler(handler) && handler.setError(null)) {
        handled = true;
    }
    if (isValidationErrorHandler(handler) && handler.setErrors(null)) {
        handled = true;
    }
    if (isConstraintViolationHandler(handler) && handler.setConstraintViolations(null)) {
        handled = true;
    }
    return handled;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Erreur basée sur une réponse HTTP d'erreur.
 */
class HTTPError extends AppError {
    /**
     * @param {?} original
     * @param {?=} message
     */
    constructor(original, message) {
        super(original, message || original.message || original.statusText, undefined, original.status >= 500);
    }
    /**
     * @return {?}
     */
    get status() {
        return this.original.status;
    }
    /**
     * @return {?}
     */
    get statusText() {
        return this.original.statusText;
    }
    /**
     * @return {?}
     */
    get headers() {
        return this.original.headers;
    }
    /**
     * @return {?}
     */
    get url() {
        return this.original.url;
    }
    /**
     * @return {?}
     */
    get error() {
        return this.original.error;
    }
}
/**
 * Erreur 403: Forbidden
 */
class AccessDeniedError extends HTTPError {
}
/**
 * Erreur 401: Authentication Required
 */
class AuthenticationError extends HTTPError {
}
/**
 * Erreur 400: Bad Request
 */
class BadRequestError extends HTTPError {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Erreur de validation retournée par api-platform.
 */
class ConstraintViolations extends BadRequestError {
    /**
     * @param {?} original
     * @param {?} message
     * @param {?} violations
     */
    constructor(original, message, violations) {
        super(original, message);
        this.violations = violations;
    }
    /**
     * @param {?} handler
     * @return {?}
     */
    dispatch(handler) {
        return ((isConstraintViolationHandler(handler) && handler.setConstraintViolations(this.violations)) ||
            super.dispatch(handler));
    }
    /**
     * @param {?} violations
     * @return {?}
     */
    static create(violations) {
        return new ConstraintViolations(new HttpErrorResponse({
            status: 400,
            statusText: 'Constraint violations',
        }), 'Constraint violations', violations);
    }
}
if (false) {
    /** @type {?} */
    ConstraintViolations.prototype.violations;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Rapport d'erreur de validation
 * @record
 */
function ConstraintViolation() { }
if (false) {
    /** @type {?} */
    ConstraintViolation.prototype.propertyPath;
    /** @type {?} */
    ConstraintViolation.prototype.message;
}
/**
 * Types de gestionnaires d'erreurs.
 * @record
 */
function ErrorMessageHandler() { }
if (false) {
    /**
     * @param {?} message
     * @return {?}
     */
    ErrorMessageHandler.prototype.setError = function (message) { };
}
/**
 * @record
 */
function ConstraintViolationHandler() { }
if (false) {
    /**
     * @param {?} violations
     * @return {?}
     */
    ConstraintViolationHandler.prototype.setConstraintViolations = function (violations) { };
}
/**
 * @record
 */
function ValidationErrorHandler() { }
if (false) {
    /**
     * @param {?} errors
     * @return {?}
     */
    ValidationErrorHandler.prototype.setErrors = function (errors) { };
}
/**
 * @record
 */
function ErrorLogger() { }
if (false) {
    /**
     * @param {?} message
     * @param {?} name
     * @param {?=} stack
     * @param {?=} context
     * @return {?}
     */
    ErrorLogger.prototype.logError = function (message, name, stack, context) { };
}
/**
 * @record
 */
function ErrorClearer() { }
if (false) {
    /**
     * @return {?}
     */
    ErrorClearer.prototype.clearErrors = function () { };
}
/**
 * @record
 */
function DispatchableError() { }
if (false) {
    /** @type {?} */
    DispatchableError.prototype.isNullError;
    /**
     * @param {?} handler
     * @return {?}
     */
    DispatchableError.prototype.dispatch = function (handler) { };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
const ɵ0 = /**
 * @param {?} handler
 * @return {?}
 */
(handler) => {
    return clearErrors(handler);
};
/**
 * Singleton d'erreur null.
 * @type {?}
 */
const NullError = {
    isNullError: true,
    name: 'NullError',
    message: 'No error',
    dispatch: (ɵ0),
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Transforme une valeur en erreur.
 *
 * Les informations intéressantes d'HttpErrorResponse et Error sont conservés.
 * @param {?} err
 * @return {?}
 */
function normalizeError(err) {
    if (err instanceof AppError || err === NullError) {
        return err;
    }
    if (err instanceof HttpErrorResponse) {
        return normalizeHttpErrorResponse(err);
    }
    if (err instanceof Error) {
        return new AppError(err, err.message, err.stack, false);
    }
    if (isEmpty(err)) {
        return NullError;
    }
    if (typeof err === 'object') {
        return new AppError(err, extractMessage(err), err.stack, err.transient || false);
    }
    return new AppError(err, extractMessage(err));
}
/**
 * @param {?} err
 * @return {?}
 */
function normalizeHttpErrorResponse(err) {
    /** @type {?} */
    const body = 'error' in err ? err.error : {};
    /** @type {?} */
    const message = extractMessage(body) || extractMessage(err);
    switch (err.status) {
        case 400:
            if (has(body, 'violations') && isArray(body.violations)) {
                return new ConstraintViolations(err, message, body.violations);
            }
            return new BadRequestError(err, message);
        case 401:
            return new AuthenticationError(err, message);
        case 403:
            return new AccessDeniedError(err, message);
        default:
            return new HTTPError(err, message);
    }
}
/**
 * @param {?} err
 * @return {?}
 */
function extractMessage(err) {
    if (!err) {
        return undefined;
    }
    switch (typeof err) {
        case 'object':
            if ('message' in err) {
                return err.message;
            }
            if ('hydra:description' in err) {
                return err['hydra:description'];
            }
            if ('hydra:title' in err) {
                return err['hydra:title'];
            }
            if ('error' in err) {
                return extractMessage(err.error);
            }
            break;
        case 'string':
        case 'number':
            return `${err}`;
    }
    return undefined;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Erreur émise lorsqu'une exception ne peut pas être dispatchée.
 */
class UnhandledError extends AppError {
    /**
     * @param {?} original
     */
    constructor(original) {
        super(original, `Unhandled error: ${original.message}`, original.stack, false);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ValidationError extends AppError {
    /**
     * @param {?} errors
     */
    constructor(errors) {
        super(errors, errors.message || 'Validation error');
        this.errors = errors;
    }
    /**
     * @param {?} handler
     * @return {?}
     */
    dispatch(handler) {
        return (isValidationErrorHandler(handler) && handler.setErrors(this.errors)) || super.dispatch(handler);
    }
}
if (false) {
    /** @type {?} */
    ValidationError.prototype.errors;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AccessDeniedError, AppError, AuthenticationError, BadRequestError, ConstraintViolations, ErrorHandlerService, ErrorModule, GenericErrorComponent, HTTPError, NotFoundErrorComponent, NullError, UnhandledError, ValidationError, clearErrors, isConstraintViolationHandler, isDispatchableError, isErrorClearer, isErrorHandler, isErrorLogger, isErrorMessageHandler, isValidationErrorHandler, normalizeError };
//# sourceMappingURL=error-types.js.map
